

using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.Reflection;

namespace HelloTxt
{
    /// <summary>HelloTxt Wrapper
    /// </summary>
    public class HelloTxtApi
    {
        public HelloTxtApi() { }
        public HelloTxtApi(string user_application_key)
        {
            this.user_application_key = user_application_key;
        }

        /// <summary>HelloTxt Developer API Key.</summary>
		[System.Reflection.Obfuscation(Feature="StringEncryption")]
        private static string api_key = "B8XjgZr1srkkcIyL";

        /// <summary>User Application Key.
        /// The end user will need to enter this. see http://ping.fm/key/
        /// </summary>
        public string user_application_key;

        #region XML Serialization Classes

        public class Service
        {
            [XmlAttribute("id")]    
            public string ID;
            [XmlAttribute("name")]  
            public string Name;
            [XmlElement("code")]
            public string Code;
            [XmlElement("nick")]
            public string Nicknname;
            [XmlElement("checked")]
            public string Checked;
            [XmlElement("inhome")]
            public string InHome;
            [XmlElement("friend")]
            public string Friend;
            [XmlElement("colleague")]
            public string Colleague;
            [XmlElement("tags")]
            public string Tags;

            public override string ToString()
            {
                if (!string.IsNullOrEmpty(Name))
                    return string.Format("{0} [{1}] ({2})", Name, ID, Nicknname);
                else
                    return base.ToString();
            }
        }

        [XmlRoot("rsp")]
        public class BaseResponse
        {
            [XmlAttribute("status")]    public string Status;
            [XmlElement("method")]      public string Method;
            [XmlElement("message")]     public string Message;
        }

        [XmlRoot("rsp")]
        public class ServicesResponse : BaseResponse
        {
            [XmlArray("services"), XmlArrayItem("service", typeof(Service))]
            public List<Service> Services;
        }

        [XmlRoot("rsp")]
        public class ValidateResponse : BaseResponse
        {
            [XmlElement("nick")]
            public string Nickname;

            [XmlElement("name")]
            public string Name;
        }


        private bool SerializeObject(XmlTextWriter writer, object o)
        {
            try
            {
                XmlSerializer xs = new XmlSerializer(o.GetType());
                XmlSerializerNamespaces ns = new XmlSerializerNamespaces();
                ns.Add("", ""); //remove the xmlns tags.
                xs.Serialize(writer, o, ns);
                return true;
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
                return false;
            }
        }
        private object DeserializeObject(XmlReader reader, Type type)
        {
            try
            {
                XmlSerializer xs = new XmlSerializer(type);
                return xs.Deserialize(reader);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
                DeserializeException = ex;
                return null;
            }
        }

        #endregion

        private HttpWebRequest CreateHttpWebRequest(string url)
        {
            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(url);
            request.Headers.Add("Accept-Language", "en-us");
            request.Accept = "text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5";
			request.UserAgent = Toth.WLW.Plugins.ThisAssembly.AssemblyProduct + " " + Toth.WLW.Plugins.ThisAssembly.AssemblyVersion;
            return request;
        }

        private string GetWebResponse(string url, string PostData)
        {
            string html = "";
            try
            {
                HttpWebRequest request = CreateHttpWebRequest(url);
                request.Method = "POST";
                request.ContentType = "application/x-www-form-urlencoded";
                byte[] b = Encoding.Default.GetBytes(PostData);
                request.GetRequestStream().Write(b, 0, b.Length);
                request.GetRequestStream().Close();
                request.Timeout = 10000;
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                StreamReader sr = new StreamReader(response.GetResponseStream());
                sr.BaseStream.ReadTimeout = 10000;
                html = sr.ReadToEnd();
                sr.Close();
                response.Close();
            }
            catch (Exception ex)
            {
                WebException = ex;
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
            return html;
        }

        private BaseResponse mLastResponse;
        /// <summary>Gets the last PingResponse</summary>
        public BaseResponse LastResponse { get { return mLastResponse; } }

        /// <summary>If a web server error occurs, the exception will be stored here for reference.</summary>
        public Exception WebException;

        /// <summary>If an error occurs during deserialization, the exception will be stored here for reference.</summary>
        public Exception DeserializeException;

        /// <summary>Clears the LastResponse, WebException, and DeserializeException</summary>
        public void Reset()
        {
            mLastResponse = null;
            WebException = null;
            DeserializeException = null;
        }

        /// <summary>Validates the given users application key.</summary>
        public ValidateResponse Validate()
        {
            string url = "http://hellotxt.com/api/v1/method/user.validate";
            string postdata = "app_key={0}&user_key={1}";
            postdata = string.Format(postdata, api_key, user_application_key);
            string response = GetWebResponse(url, postdata);
            XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
            ValidateResponse r = (ValidateResponse)DeserializeObject(xr, typeof(ValidateResponse));
            xr.Close();
            mLastResponse = r;
            return r;
        }

        /// <summary>Returns a list of services the particular user has set up through HelloTxt.</summary>
        public ServicesResponse GetServices()
        {
            string url = "http://hellotxt.com/api/v1/method/user.services";
            string postdata = "app_key={0}&user_key={1}";
            postdata = string.Format(postdata, api_key, user_application_key);
            string response = GetWebResponse(url, postdata);
            XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
            ServicesResponse r = (ServicesResponse)DeserializeObject(xr, typeof(ServicesResponse));
            xr.Close();
            mLastResponse = r;
            return r;
        }

        /// <summary>Posts a message to the users Ping.fm services.</summary>
        public BaseResponse Post(string Body)
        {
            return Post(null, Body, false);
        }
        public BaseResponse Post(string Title, string Body)
        {
            return Post(Title, Body, false);
        }
        public BaseResponse Post(string Title, string Body, bool debug)
        {
			return Post(Title, Body, debug, null, null);
        }

		public BaseResponse Post(string Title, string Body, bool debug, string Group, string Networks)
		{
            string url = "http://hellotxt.com/api/v1/method/user.post";
			string postdata = "app_key={0}&user_key={1}";
			postdata = string.Format(postdata, api_key, user_application_key);
			if (!string.IsNullOrEmpty(Title)) postdata += "&title=" + UrlEncode(Title);
			if (!string.IsNullOrEmpty(Body)) postdata += "&body=" + UrlEncode(Body);
			if (!string.IsNullOrEmpty(Group)) postdata += "&group=" + UrlEncode(Group);
			if (!string.IsNullOrEmpty(Networks)) postdata += "&networks=" + UrlEncode(Networks);

			if (debug) postdata += "&debug=1";
			string response = GetWebResponse(url, postdata);
			XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
			BaseResponse r = (BaseResponse)DeserializeObject(xr, typeof(BaseResponse));
			xr.Close();
			mLastResponse = r;
			return r;
		}

        public static string Base64Encode(string ToEncode)
        {
            if (string.IsNullOrEmpty(ToEncode)) return ToEncode;
            try
            {
                //this method is a little more work, but supports unicode.
                char[] chars = ToEncode.ToCharArray();
                byte[] bytes = Encoding.UTF8.GetBytes(chars);
                return Convert.ToBase64String(bytes);
            }
            catch
            {
                return ToEncode;
            }
        }

        public static string Base64Decode(string ToDecode)
        {
            try
            {
                return Encoding.Default.GetString(Convert.FromBase64String(ToDecode));
            }
            catch { return ToDecode; }
        }

        private static string UrlEncode(string text)
        {
            return System.Web.HttpUtility.UrlEncode(text, Encoding.Default);
        }

    }
}
