/*
    Copyright  2008, Ping.fm Inc.
    All rights reserved.
    http://ping.fm/

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

    - Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer. 

    - Neither the name Ping.fm, nor the names of its contributors
       may be used to endorse or promote products derived from this
       software without specific prior written permission. 

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.Reflection;

namespace PingFM
{
    /// <summary>Ping.fm API wrapper for C#.
    /// Created by Adam Duffy (adam@ping.fm) 06/18/2008. 
    /// Modified by Adam Toth 03/01/2009
    /// </summary>
    public class PingFMApi
    {
        public PingFMApi() { }
        public PingFMApi(string user_application_key)
        {
            this.user_application_key = user_application_key;
        }

        public enum PingEncoding { URL, Base64 };

        /// <summary>Ping.fm Developer API Key.  See http://ping.fm/developers/</summary>
        [System.Reflection.Obfuscation(Feature = "StringEncryption")]
        private static string api_key = "efe2cf95d638369e149a175ad2ea4271";

        /// <summary>User Application Key.
        /// The end user will need to enter this. see http://ping.fm/key/
        /// </summary>
        public string user_application_key;

        //#region XML Serialization Classes

        //public class ServiceMethods
        //{
        //    [XmlAttribute("id")]    public string ID;
        //    [XmlAttribute("name")]  public string Name;
        //    [XmlElement("methods")] public string Methods;
        //    public override string ToString()
        //    {
        //        if (!string.IsNullOrEmpty(Name))
        //            return Name + " [" + ID + "]";
        //        else
        //            return base.ToString();
        //    }
        //}
        //public class Service
        //{
        //    [XmlAttribute("id")]    public string ID;
        //    [XmlAttribute("name")]  public string Name;
        //    [XmlElement("methods")] public string Methods;
        //    public override string ToString()
        //    {
        //        if (!string.IsNullOrEmpty(Name))
        //            return Name + " [" + ID + "]";
        //        else
        //            return base.ToString();
        //    }
        //}
        //public class Trigger
        //{
        //    [XmlAttribute("id")]     public string ID;
        //    [XmlAttribute("method")] public string Method;
        //    [XmlArray("services"), XmlArrayItem("service", typeof(Service))]
        //    public List<Service> Services = new List<Service>();
        //    public override string ToString()
        //    {
        //        if (!string.IsNullOrEmpty(ID))
        //            return ID + " (" + Method + ")";
        //        else
        //            return base.ToString();
        //    }
        //}
        //public class Message
        //{
        //    public struct PingDate
        //    {
        //        [XmlAttribute("rfc")]  public string Rfc;
        //        [XmlAttribute("unix")] public string Unix;
        //        public override string ToString()
        //        {
        //            if (!string.IsNullOrEmpty(Rfc))
        //                return Rfc;
        //            else
        //                return base.ToString();
        //        }
        //    }
        //    public struct PingContent
        //    {
        //        [XmlElement("title")] public string Title;
        //        [XmlElement("body")]  public string Body;
        //        public override string ToString()
        //        {
        //            if (!string.IsNullOrEmpty(Title))
        //                return Title;
        //            else if (!string.IsNullOrEmpty(Body))
        //                return Body;
        //            else
        //                return base.ToString();
        //        }
        //    }
        //    [XmlAttribute("id")]     public string ID;
        //    [XmlAttribute("method")] public string Method;
        //    [XmlElement("date")]     public PingDate Date;
        //    [XmlElement("content")]  public PingContent Content;
        //    [XmlArray("services"), XmlArrayItem("service", typeof(Service))]
        //    public List<Service> Services = new List<Service>();
        //    public void Decode()
        //    {
        //        Content.Body = Base64Decode(Content.Body);
        //        Content.Title = Base64Decode(Content.Title);
        //    }
        //    public override string ToString()
        //    {
        //        if (!string.IsNullOrEmpty(Method))
        //            return Method;
        //        else
        //            return base.ToString();
        //    }
        //}

        //[XmlRoot("rsp")]
        //public class PingResponse
        //{
        //    [XmlAttribute("status")]    public string Status;
        //    [XmlElement("transaction")] public string Transaction;
        //    [XmlElement("method")]      public string Method;
        //    [XmlElement("message")]     public string Message;
        //}

        //[XmlRoot("rsp")]
        //public class ServicesResponse : PingResponse
        //{
        //    [XmlArray("services"), XmlArrayItem("service", typeof(ServiceMethods))]
        //    public List<ServiceMethods> Services;
        //}

        //[XmlRoot("rsp")]
        //public class TriggerResponse : PingResponse
        //{
        //    [XmlArray("triggers"), XmlArrayItem("trigger", typeof(Trigger))]
        //    public List<Trigger> Triggers;
        //}

        //[XmlRoot("rsp")]
        //public class LatestResponse : PingResponse
        //{
        //    [XmlArray("messages"), XmlArrayItem("message", typeof(Message))]
        //    public List<Message> Messages = new List<Message>();
        //    public void DecodeMessages()
        //    {
        //        if (Messages == null) return;
        //        foreach (Message m in Messages) 
        //            m.Decode();
        //    }
        //}

        //[XmlRoot("rsp")]
        //public class MessageResponse : PingResponse
        //{
        //    //.net has a problem deserializing this one.
        //    //could un-inherit the class, but i'll leave the bug here.
        //    [XmlElement("message")]
        //    public new Message Message;
        //}

        //private bool SerializeObject(XmlTextWriter writer, object o)
        //{
        //    try
        //    {
        //        XmlSerializer xs = new XmlSerializer(o.GetType());
        //        XmlSerializerNamespaces ns = new XmlSerializerNamespaces();
        //        ns.Add("", ""); //remove the xmlns tags.
        //        xs.Serialize(writer, o, ns);
        //        return true;
        //    }
        //    catch (Exception ex)
        //    {
        //        System.Diagnostics.Debug.WriteLine(ex.Message);
        //        return false;
        //    }
        //}
        //private object DeserializeObject(XmlReader reader, Type type)
        //{
        //    try
        //    {
        //        XmlSerializer xs = new XmlSerializer(type);
        //        return xs.Deserialize(reader);
        //    }
        //    catch (Exception ex)
        //    {
        //        System.Diagnostics.Debug.WriteLine(ex.Message);
        //        DeserializeException = ex;
        //        return null;
        //    }
        //}

        //#endregion

        #region XML Serialization Classes

        public class ServiceMethods
        {
            [XmlAttribute("id")]
            public string ID;
            [XmlAttribute("name")]
            public string Name;
            [XmlElement("trigger")]
            public string Trigger;
            [XmlElement("url")]
            public string Url;
            [XmlElement("icon")]
            public string Icon;
            [XmlElement("methods")]
            public string Methods;
            public override string ToString()
            {
                if (!string.IsNullOrEmpty(Name))
                    return string.Format("{0} - {1}", Trigger, Name);
                else
                    return base.ToString();
            }
        }
        public class Service
        {
            [XmlAttribute("id")]
            public string ID;
            [XmlAttribute("name")]
            public string Name;
            [XmlElement("methods")]
            public string Methods;
            public override string ToString()
            {
                if (!string.IsNullOrEmpty(Name))
                    return Name + " [" + ID + "]";
                else
                    return base.ToString();
            }
        }
        public class Trigger
        {
            [XmlAttribute("id")]
            public string ID;
            [XmlAttribute("method")]
            public string Method;
            [XmlArray("services"), XmlArrayItem("service", typeof(Service))]
            public List<Service> Services = new List<Service>();
            public override string ToString()
            {
                if (!string.IsNullOrEmpty(ID))
                    return ID + " (" + Method + ")";
                else
                    return base.ToString();
            }
        }
        public class Message
        {
            public struct PingDate
            {
                [XmlAttribute("rfc")]
                public string Rfc;
                [XmlAttribute("unix")]
                public string Unix;
                public override string ToString()
                {
                    if (!string.IsNullOrEmpty(Rfc))
                        return Rfc;
                    else
                        return base.ToString();
                }
            }
            public struct PingContent
            {
                [XmlElement("title")]
                public string mTitle;
                [XmlElement("body")]
                public string mBody;
                [XmlIgnore()]
                public string Title { get { return Base64Decode(mTitle); } }
                [XmlIgnore()]
                public string Body { get { return Base64Decode(mBody); } }
                public override string ToString()
                {
                    if (!string.IsNullOrEmpty(Title))
                        return Title;
                    else if (!string.IsNullOrEmpty(Body))
                        return Body;
                    else
                        return base.ToString();
                }
            }
            [XmlAttribute("id")]
            public string ID;
            [XmlAttribute("method")]
            public string Method;
            [XmlElement("date")]
            public PingDate Date;
            [XmlElement("content")]
            public PingContent Content;
            [XmlArray("services"), XmlArrayItem("service", typeof(Service))]
            public List<Service> Services = new List<Service>();
            public override string ToString()
            {
                if (!string.IsNullOrEmpty(Method))
                    return Method;
                else
                    return base.ToString();
            }
        }

        [XmlRoot("rsp")]
        public class PingResponse
        {
            [XmlAttribute("status")]
            public string Status;
            [XmlElement("transaction")]
            public string Transaction;
            [XmlElement("method")]
            public string Method;
            [XmlElement("message")]
            public string Message;
        }

        [XmlRoot("rsp")]
        public class ServicesResponse : PingResponse
        {
            [XmlArray("services"), XmlArrayItem("service", typeof(ServiceMethods))]
            public List<ServiceMethods> Services;
        }

        [XmlRoot("rsp")]
        public class TriggerResponse : PingResponse
        {
            [XmlArray("triggers"), XmlArrayItem("trigger", typeof(Trigger))]
            public List<Trigger> Triggers;
        }

        [XmlRoot("rsp")]
        public class LatestResponse : PingResponse
        {
            [XmlArray("messages"), XmlArrayItem("message", typeof(Message))]
            public List<Message> Messages = new List<Message>();
        }

        [XmlRoot("rsp")]
        public class MessageResponse : PingResponse
        {
            //.net has a problem deserializing this one.
            //could un-inherit the class, but i'll leave the bug here.
            [XmlElement("message")]
            public new Message Message;
        }

        [XmlRoot("rsp")]
        public class KeyResponse : PingResponse
        {
            [XmlElement("key")]
            public string Key;
        }

        [XmlRoot("rsp")]
        public class UrlCreateResponse : PingResponse
        {
            [XmlElement("short_url")]
            public string ShortURL;
        }
        [XmlRoot("rsp")]
        public class UrlResolveResponse : PingResponse
        {
            [XmlElement("url")]
            public string mURL;
            [XmlIgnore()]
            public string URL { get { return Base64Decode(mURL); } }
        }

        private bool SerializeObject(XmlTextWriter writer, object o)
        {
            try
            {
                XmlSerializer xs = new XmlSerializer(o.GetType());
                XmlSerializerNamespaces ns = new XmlSerializerNamespaces();
                ns.Add("", ""); //remove the xmlns tags.
                xs.Serialize(writer, o, ns);
                return true;
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
                return false;
            }
        }
        private object DeserializeObject(XmlReader reader, Type type)
        {
            try
            {
                XmlSerializer xs = new XmlSerializer(type);
                return xs.Deserialize(reader);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
                DeserializeException = ex;
                return null;
            }
        }

        #endregion

        /// <summary>Stores all the possible values for an outgoing message</summary>
        public class OutgoingMessage
        {
            public OutgoingMessage() { }
            public OutgoingMessage(string Body) { this.Body = Body; }
            public OutgoingMessage(string Title, string Body) { this.Title = Title; this.Body = Body; }
            [XmlElement("Method")]
            public string Method;
            [XmlElement("Trigger")]
            public string Trigger;
            [XmlElement("Service")]
            public string Service;
            [XmlElement("Title")]
            public string Title;
            [XmlElement("Body")]
            public string Body;
            [XmlElement("Location")]
            public string Location;
            [XmlElement("Tags")]
            public string Tags;
            [XmlElement("Mood")]
            public string Mood;
            [XmlElement("Exclude")]
            public string Exclude;
            [XmlArray("MediaFilenames"), XmlArrayItem("Filename", typeof(string))]
            public List<string> MediaFilenames = new List<string>();
            [XmlElement("Debug")]
            public bool Debug;
            [XmlElement("Encoding")]
            public PingEncoding Encoding;
        }

        private static HttpWebRequest CreateHttpWebRequest(string url)
        {
            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(url);
            request.Headers.Add("Accept-Language", "en-us");
            request.Accept = "text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5";
            request.UserAgent = Toth.WLW.Plugins.ThisAssembly.AssemblyProduct + " " + Toth.WLW.Plugins.ThisAssembly.AssemblyVersion;
            request.ContentType = "application/x-www-form-urlencoded";
            return request;
        }

        //private string GetWebResponse(string url, string PostData)
        //{
        //    string html = "";
        //    try
        //    {
        //        HttpWebRequest request = CreateHttpWebRequest(url);
        //        request.Method = "POST";
        //        request.ContentType = "application/x-www-form-urlencoded";
        //        byte[] b = Encoding.Default.GetBytes(PostData);
        //        request.GetRequestStream().Write(b, 0, b.Length);
        //        request.GetRequestStream().Close();
        //        request.Timeout = 10000;
        //        HttpWebResponse response = (HttpWebResponse)request.GetResponse();
        //        StreamReader sr = new StreamReader(response.GetResponseStream());
        //        sr.BaseStream.ReadTimeout = 10000;
        //        html = sr.ReadToEnd();
        //        sr.Close();
        //        response.Close();
        //    }
        //    catch (Exception ex)
        //    {
        //        WebException = ex;
        //        System.Diagnostics.Debug.WriteLine(ex.Message);
        //    }
        //    return html;
        //}

        private string GetWebResponse(string url, string PostData)
        {
            string html = "";
            try
            {
#if DEBUG
                System.Diagnostics.Debug.WriteLine(url);
                System.Diagnostics.Debug.WriteLine(PostData);
#endif
                HttpWebRequest request = CreateHttpWebRequest(url);
                request.Method = "POST";
                request.ContentType = "application/x-www-form-urlencoded";
                byte[] b = Encoding.Default.GetBytes(PostData);
                request.ContentLength = b.Length;
                Stream s = request.GetRequestStream();
                s.Write(b, 0, b.Length);
                s.Close();
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                StreamReader sr = new StreamReader(response.GetResponseStream());
                html = sr.ReadToEnd();
                sr.Close();
                response.Close();
#if DEBUG
                System.Diagnostics.Debug.WriteLine(html);
#endif
            }
            catch (Exception ex)
            {
                WebException = ex;
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
            return html;
        }


        private PingResponse mLastResponse;
        /// <summary>Gets the last PingResponse</summary>
        public PingResponse LastResponse { get { return mLastResponse; } }

        /// <summary>If a web server error occurs, the exception will be stored here for reference.</summary>
        public Exception WebException;

        /// <summary>If an error occurs during deserialization, the exception will be stored here for reference.</summary>
        public Exception DeserializeException;

        /// <summary>Clears the LastResponse, WebException, and DeserializeException</summary>
        public void Reset()
        {
            mLastResponse = null;
            WebException = null;
            DeserializeException = null;
        }

        ///// <summary>Validates the given users application key.</summary>
        //public PingResponse Validate()
        //{
        //    string url = "http://api.ping.fm/v1/user.validate";
        //    string postdata = "api_key={0}&user_app_key={1}";
        //    postdata = string.Format(postdata, api_key, user_application_key);
        //    string response = GetWebResponse(url, postdata);
        //    XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
        //    PingResponse r = (PingResponse)DeserializeObject(xr, typeof(PingResponse));
        //    xr.Close();
        //    mLastResponse = r;
        //    return r;
        //}

        ///// <summary>Returns a list of services the particular user has set up through Ping.fm.</summary>
        //public ServicesResponse GetServices()
        //{
        //    string url = "http://api.ping.fm/v1/user.services";
        //    string postdata = "api_key={0}&user_app_key={1}";
        //    postdata = string.Format(postdata, api_key, user_application_key);
        //    string response = GetWebResponse(url, postdata);
        //    XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
        //    ServicesResponse r = (ServicesResponse)DeserializeObject(xr, typeof(ServicesResponse));
        //    xr.Close();
        //    mLastResponse = r;
        //    return r;
        //}

        ///// <summary>Validates the given users application key.</summary>
        //public TriggerResponse GetTriggers()
        //{
        //    string url = "http://api.ping.fm/v1/user.triggers";
        //    string postdata = "api_key={0}&user_app_key={1}";
        //    postdata = string.Format(postdata, api_key, user_application_key);
        //    string response = GetWebResponse(url, postdata);
        //    XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
        //    TriggerResponse r = (TriggerResponse)DeserializeObject(xr, typeof(TriggerResponse));
        //    xr.Close();
        //    mLastResponse = r;
        //    return r;
        //}

        /// <summary>Validates the given users application key.</summary>
        public PingResponse Validate()
        {
            string url = "http://api.ping.fm/v1/user.validate";
            string postdata = "api_key={0}&user_app_key={1}";
            postdata = string.Format(postdata, api_key, user_application_key);
            string response = GetWebResponse(url, postdata);
            XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
            PingResponse r = (PingResponse)DeserializeObject(xr, typeof(PingResponse));
            xr.Close();
            mLastResponse = r;
            return r;
        }

        /// <summary>Returns a list of services the particular user has set up through Ping.fm.</summary>
        public ServicesResponse GetServices()
        {
            string url = "http://api.ping.fm/v1/user.services";
            string postdata = "api_key={0}&user_app_key={1}";
            postdata = string.Format(postdata, api_key, user_application_key);
            string response = GetWebResponse(url, postdata);
            XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
            ServicesResponse r = (ServicesResponse)DeserializeObject(xr, typeof(ServicesResponse));
            xr.Close();
            mLastResponse = r;
            return r;
        }

        /// <summary>Validates the given users application key.</summary>
        public TriggerResponse GetTriggers()
        {
            string url = "http://api.ping.fm/v1/user.triggers";
            string postdata = "api_key={0}&user_app_key={1}";
            postdata = string.Format(postdata, api_key, user_application_key);
            string response = GetWebResponse(url, postdata);
            XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
            TriggerResponse r = (TriggerResponse)DeserializeObject(xr, typeof(TriggerResponse));
            xr.Close();
            mLastResponse = r;
            return r;
        }

        /// <summary>Returns the last 25 messages a user has posted through Ping.fm.</summary>
        public LatestResponse GetLatest()
        {
            return GetLatest(-1, null);
        }
        /// <summary>Returns the last X messages a user has posted through Ping.fm.</summary>
        /// <param name="limit">Number of messages to query</param>
        /// <param name="order">Order of results (ASC/DESC)</param>
        public LatestResponse GetLatest(int limit, string order)
        {
            string url = "http://api.ping.fm/v1/user.latest";
            string postdata = "api_key={0}&user_app_key={1}";
            postdata = string.Format(postdata, api_key, user_application_key);
            if (limit > -1) postdata += "&limit=" + limit.ToString();
            if (!string.IsNullOrEmpty(order)) postdata += "&order=" + order;
            string response = GetWebResponse(url, postdata);
            XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
            LatestResponse r = (LatestResponse)DeserializeObject(xr, typeof(LatestResponse));
            xr.Close();
            //if (r != null) r.DecodeMessages();
            mLastResponse = r;
            return r;
        }

        /// <summary>Returns data for the specified MessageID.</summary>
        /// <param name="MessageID">MessageID to query</param>
        [Obsolete("Use user.latest to get a recent message history", true)]
        public MessageResponse GetMessage(string MessageID)
        {
            string url = "http://api.ping.fm/v1/user.message";
            string postdata = "api_key={0}&user_app_key={1}&message_id={2}";
            postdata = string.Format(postdata, api_key, user_application_key, MessageID);
            string response = GetWebResponse(url, postdata);
            XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
            MessageResponse r = (MessageResponse)DeserializeObject(xr, typeof(MessageResponse));
            xr.Close();
            //if (r != null && r.Message != null) r.Message.Decode();
            mLastResponse = r;
            return r;
        }

        /// <summary>Allows for flexible encoding methods</summary>
        private delegate string EncodeDelegate(string s);


        ///// <summary>Posts a message to the users Ping.fm services.</summary>
        //public PingResponse Post(string Body)
        //{
        //    return Post(null, null, Body, false);
        //}
        //public PingResponse Post(string Method, string Body)
        //{
        //    return Post(Method, null, Body, false);
        //}
        //public PingResponse Post(string Method, string Title, string Body)
        //{
        //    return Post(Method, Title, Body, false);
        //}
        //public PingResponse Post(string Method, string Title, string Body, bool debug)
        //{
        //    return Post(Method, Title, Body, debug, null, null, null);
        //}

        //public PingResponse Post(string Method, string Title, string Body, bool debug, string Service, string ServicesToExclude, string Location)
        //{
        //    string url = "http://api.ping.fm/v1/user.post";
        //    string postdata = "api_key={0}&user_app_key={1}&post_method={2}";
        //    if (string.IsNullOrEmpty(Method)) Method = "default";
        //    postdata = string.Format(postdata, api_key, user_application_key, Method);
        //    if (!string.IsNullOrEmpty(Title)) postdata += "&title=" + UrlEncode(Title);
        //    if (!string.IsNullOrEmpty(Body)) postdata += "&body=" + UrlEncode(Body);
        //    if (!string.IsNullOrEmpty(Service)) postdata += "&service=" + UrlEncode(Service);
        //    if (!string.IsNullOrEmpty(ServicesToExclude)) postdata += "&exclude=" + UrlEncode(ServicesToExclude);
        //    if (!string.IsNullOrEmpty(Location)) postdata += "&location=" + UrlEncode(Location);

        //    if (debug) postdata += "&debug=1";
        //    string response = GetWebResponse(url, postdata);
        //    XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
        //    PingResponse r = (PingResponse)DeserializeObject(xr, typeof(PingResponse));
        //    xr.Close();
        //    mLastResponse = r;
        //    return r;
        //}

        ///// <summary>Posts a message to the users Ping.fm services using one of their custom triggers.</summary>
        //public PingResponse TPost(string Trigger, string Body)
        //{
        //    return TPost(Trigger, null, Body, false);
        //}
        //public PingResponse TPost(string Trigger, string Title, string Body)
        //{
        //    return TPost(Trigger, Title, Body, false);
        //}
        //public PingResponse TPost(string Trigger, string Title, string Body, bool debug)
        //{
        //    return TPost(Trigger, Title, Body, debug, null, null);
        //}
        //public PingResponse TPost(string Trigger, string Title, string Body, bool debug, string ServicesToExclude, string Location)
        //{
        //    string url = "http://api.ping.fm/v1/user.tpost";
        //    string postdata = "api_key={0}&user_app_key={1}";
        //    postdata = string.Format(postdata, api_key, user_application_key);
        //    if (!string.IsNullOrEmpty(Trigger)) postdata += "&trigger=" + Trigger;
        //    if (!string.IsNullOrEmpty(Title)) postdata += "&title=" + UrlEncode(Title);
        //    if (!string.IsNullOrEmpty(Body)) postdata += "&body=" + UrlEncode(Body);
        //    if (!string.IsNullOrEmpty(ServicesToExclude)) postdata += "&exclude=" + UrlEncode(ServicesToExclude);
        //    if (!string.IsNullOrEmpty(Location)) postdata += "&location=" + UrlEncode(Location);
        //    if (debug) postdata += "&debug=1";
        //    string response = GetWebResponse(url, postdata);
        //    XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
        //    PingResponse r = (PingResponse)DeserializeObject(xr, typeof(PingResponse));
        //    xr.Close();
        //    mLastResponse = r;
        //    return r;
        //}

        /// <summary>Posts a message to the users Ping.fm services.</summary>
        public PingResponse Post(OutgoingMessage message)
        {
            string url = "http://api.ping.fm/v1/user.post";
            string postdata = "api_key={0}&user_app_key={1}";
            string method = message.Method;
            EncodeDelegate encoder = UrlEncode;
            postdata = string.Format(postdata, api_key, user_application_key, method);
            if (message.Encoding == PingEncoding.Base64)
            {
                postdata += "&encoding=base64";
                encoder = Base64Encode;
            }
            if (string.IsNullOrEmpty(message.Trigger))
            {
                if (string.IsNullOrEmpty(method)) method = "default";
                postdata += "&post_method=" + UrlEncode(method);
            }
            else
            {
                url = "http://api.ping.fm/v1/user.tpost";
                postdata += "&trigger=" + UrlEncode(message.Trigger);
            }
            if (!string.IsNullOrEmpty(message.Title)) postdata += "&title=" + encoder(message.Title);
            if (!string.IsNullOrEmpty(message.Body)) postdata += "&body=" + encoder(message.Body);
            if (!string.IsNullOrEmpty(message.Service)) postdata += "&service=" + UrlEncode(message.Service);
            if (!string.IsNullOrEmpty(message.Exclude)) postdata += "&exclude=" + UrlEncode(message.Exclude);
            if (!string.IsNullOrEmpty(message.Location)) postdata += "&location=" + encoder(message.Location);
            if (!string.IsNullOrEmpty(message.Tags)) postdata += "&tags=" + encoder(message.Tags);
            if (!string.IsNullOrEmpty(message.Mood)) postdata += "&mood=" + encoder(message.Mood);
            if (message.Debug) postdata += "&debug=1";
            if (message.MediaFilenames != null)
                foreach (string s in message.MediaFilenames)
                    postdata += "&media=" + Base64Encode(GetMediaContent(s));
            string response = GetWebResponse(url, postdata);
            XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
            PingResponse r = (PingResponse)DeserializeObject(xr, typeof(PingResponse));
            xr.Close();
            mLastResponse = r;
            return r;
        }

        /// <summary>Posts a message to the users Ping.fm services using one of their custom triggers.</summary>
        [Obsolete("Use Post with message.trigger value set", true)]
        public PingResponse TPost(OutgoingMessage message)
        {
            //            string url = "http://api.ping.fm/v1/user.tpost";
            //            string postdata = "api_key={0}&user_app_key={1}";
            //            postdata = string.Format(postdata, api_key, user_application_key);
            //            EncodeDelegate encoder = UrlEncode;
            //            if (message.Encoding == PingEncoding.Base64)
            //            {
            //                encoder = Base64Encode;
            //                postdata += "&encoding=base64";
            //            }
            //            if (!string.IsNullOrEmpty(message.Trigger)) postdata += "&trigger=" + message.Trigger;
            //            if (!string.IsNullOrEmpty(message.Title)) postdata += "&title=" + encoder(message.Title);
            //            if (!string.IsNullOrEmpty(message.Body)) postdata += "&body=" + encoder(message.Body);
            //            if (!string.IsNullOrEmpty(message.Exclude)) postdata += "&exclude=" + UrlEncode(message.Exclude);
            //            if (!string.IsNullOrEmpty(message.Location)) postdata += "&location=" + encoder(message.Location);
            //            if (message.Debug) postdata += "&debug=1";
            //            if (message.MediaFilenames != null)
            //                foreach (string s in message.MediaFilenames)
            //                    postdata += "&media=" + Base64Encode(GetMediaContent(s));
            //#if DEBUG
            //            System.Diagnostics.Debug.WriteLine(url);
            //            System.Diagnostics.Debug.WriteLine(postdata);
            //#endif
            //            string response = GetWebResponse(url, postdata);
            //            XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
            //            PingResponse r = (PingResponse)DeserializeObject(xr, typeof(PingResponse));
            //            xr.Close();
            //            mLastResponse = r;
            //            return r;
            return null;
        }


        /// <summary>Retrieves the user's application key</summary>
        public KeyResponse GetKey(string MobileKey)
        {
            string url = "http://api.ping.fm/v1/user.key";
            string postdata = "api_key={0}&mobile_key={1}";
            postdata = string.Format(postdata, api_key, MobileKey);
            string response = GetWebResponse(url, postdata);
            XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
            KeyResponse r = (KeyResponse)DeserializeObject(xr, typeof(KeyResponse));
            xr.Close();
            mLastResponse = r;
            return r;
        }

        /// <summary>Create short Ping.fm links for URLs.</summary>
        public UrlCreateResponse UrlCreate(string LongURL)
        {
            string url = "http://api.ping.fm/v1/url.create";
            string postdata = "api_key={0}&user_app_key={1}&long_url={2}";
            postdata = string.Format(postdata, api_key, user_application_key, UrlEncode(LongURL));
            string response = GetWebResponse(url, postdata);
            XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
            UrlCreateResponse r = (UrlCreateResponse)DeserializeObject(xr, typeof(UrlCreateResponse));
            xr.Close();
            mLastResponse = r;
            return r;
        }

        /// <summary>Resolve a URL shortened by Ping.fm</summary>
        public UrlResolveResponse UrlResolve(string ShortURL)
        {
            string url = "http://api.ping.fm/v1/url.resolve";
            string postdata = "api_key={0}&user_app_key={1}&short_url={2}";
            postdata = string.Format(postdata, api_key, user_application_key, UrlEncode(ShortURL));
            string response = GetWebResponse(url, postdata);
            XmlReader xr = XmlReader.Create(new System.IO.StringReader(response));
            UrlResolveResponse r = (UrlResolveResponse)DeserializeObject(xr, typeof(UrlResolveResponse));
            xr.Close();
            mLastResponse = r;
            return r;
        }

        /// <summary>Downloads a service icon to a local file</summary>
        public static bool GetServiceIcon(string url, string LocalFile)
        {
            try
            {
                HttpWebRequest request = CreateHttpWebRequest(url);
                request.Method = "GET";
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                Stream s = response.GetResponseStream();
                System.IO.FileStream fs = new FileStream(LocalFile, FileMode.CreateNew);
                int b;
                while ((b = s.ReadByte()) != -1) fs.WriteByte((byte)b);
                fs.Close();
                s.Close();
                response.Close();
                return true;
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
            return false;
        }

        private byte[] GetMediaContent(string Filename)
        {
            byte[] b = null;
            System.IO.StreamReader sr = null;
            try
            {
                FileInfo fi = new FileInfo(Filename);
                sr = new System.IO.StreamReader(Filename);
                System.IO.BinaryReader br = new BinaryReader(sr.BaseStream);
                b = br.ReadBytes((int)fi.Length);
                br.Close();
                sr.Close();
            }
            catch
            {
                if (sr != null) sr.Close();
            }
            return b;
        }

        #region Encoding/Decoding

        public static string Base64Encode(string ToEncode)
        {
            if (string.IsNullOrEmpty(ToEncode)) return ToEncode;
            try
            {
                //this method is a little more work, but supports unicode.
                char[] chars = ToEncode.ToCharArray();
                byte[] bytes = Encoding.UTF8.GetBytes(chars);
                return Convert.ToBase64String(bytes);
            }
            catch
            {
                return ToEncode;
            }
        }

        public static string Base64Encode(byte[] ToEncode)
        {
            if (ToEncode == null) return null;
            try
            {
                return Convert.ToBase64String(ToEncode);
            }
            catch
            {
                return Encoding.Default.GetString(ToEncode, 0, ToEncode.Length);
            }
        }

        public static string Base64Decode(string ToDecode)
        {
            try
            {
                byte[] b = Convert.FromBase64String(ToDecode);
                return Encoding.Default.GetString(b, 0, b.Length);
            }
            catch { return ToDecode; }
        }

        private static string UrlEncode(string text)
        {
            //Desktop:
            return System.Web.HttpUtility.UrlEncode(text, Encoding.Default);

            //Mobile:
            //this is not completely reliable but since .net cf does not have urlencode this is close enough.
            //the Ping.fm api also allows base64 encoding of user-input values since they may have other unknown characters
            //see outgoingmessage.encoding
            //char[] chars = new char[] { '%', ';', '?', '/', ':', '#', '&', '=', '+', '$', ',', '<', '>', '~' };
            //foreach (char c in chars)
            //{
            //    if (text.IndexOf(c) > -1)
            //        text = text.Replace(c.ToString(), "%" + ((byte)c).ToString("X2"));
            //}
            //if (text.IndexOf(' ') > -1) text = text.Replace(' ', '+');
            //return text;
        }

        private static string MD5(string s)
        {
            return MD5(System.Text.Encoding.Default.GetBytes(s));
        }

        private static string MD5(byte[] bytes)
        {
            System.Security.Cryptography.MD5 md5 = System.Security.Cryptography.MD5.Create();
            byte[] b = md5.ComputeHash(bytes);
            string hex = "";
            foreach (byte b2 in b)
            {
                hex += b2.ToString("x0").PadLeft(2, '0');
            }
            md5.Clear();
            return hex;
        }

        #endregion

    }
}
